@@ -11,7 +11,7 @@ Full documentation is available at http://livestreamer.tanuki.se/.
 
 
 __title__ = "livestreamer"
-__version__ = "1.9.0"
+__version__ = "1.10.0"
 __license__ = "Simplified BSD"
 __author__ = "Christopher Rosell"
 __copyright__ = "Copyright 2011-2014 Christopher Rosell"
@@ -22,7 +22,8 @@ __credits__ = [
     "Martin Panter", "t0mm0", "Agustin Carrasco", "Andy Mikhailenko",
     "unintended", "Moritz Blanke", "Jon Bergli Heier", "Stefan Breunig",
     "papplampe", "Brian Callahan", "Eric J", "Jan Tore Morken", "JOKer",
-    "Max Nordlund", "yeeeargh"
+    "Max Nordlund", "yeeeargh", "Vitaly Evtushenko", "Che", "nixxquality",
+    "medina", "Michael Cheah"
 ]
 
 
