@@ -6,8 +6,10 @@ import signal
 import webbrowser
 
 from contextlib import closing
-from time import sleep
 from distutils.version import StrictVersion
+from functools import partial
+from itertools import chain
+from time import sleep
 
 from livestreamer import (Livestreamer, StreamError, PluginError,
                           NoPluginError)
@@ -19,12 +21,12 @@ from .compat import stdout, is_win32
 from .console import ConsoleOutput
 from .constants import CONFIG_FILES, PLUGINS_DIR, STREAM_SYNONYMS
 from .output import FileOutput, PlayerOutput
-from .utils import NamedPipe, HTTPServer, ignored, stream_to_url
+from .utils import NamedPipe, HTTPServer, ignored, progress, stream_to_url
 
 ACCEPTABLE_ERRNO = (errno.EPIPE, errno.EINVAL, errno.ECONNRESET)
 QUIET_OPTIONS = ("json", "stream_url", "subprocess_cmdline", "quiet")
 
-args = console = livestreamer = plugin = None
+args = console = livestreamer = plugin = stream_fd = None
 
 
 def check_file_output(filename, force):
@@ -202,6 +204,7 @@ def open_stream(stream):
     before opening the output.
 
     """
+    global stream_fd
 
     # Attempts to open the stream
     try:
@@ -254,57 +257,46 @@ def output_stream(stream):
     return True
 
 
-def read_stream(stream, output, prebuffer):
+def read_stream(stream, output, prebuffer, chunk_size=8192):
     """Reads data from stream and then writes it to the output."""
-
     is_player = isinstance(output, PlayerOutput)
     is_http = isinstance(output, HTTPServer)
     is_fifo = is_player and output.namedpipe
     show_progress = isinstance(output, FileOutput) and output.fd is not stdout
-    written = 0
 
-    while True:
-        try:
-            data = prebuffer or stream.read(8192)
-        except IOError as err:
-            console.logger.error("Error when reading from stream: {0}",
-                                 str(err))
-            break
+    stream_iterator = chain(
+        [prebuffer],
+        iter(partial(stream.read, chunk_size), b"")
+    )
+    if show_progress:
+        stream_iterator = progress(stream_iterator,
+                                   prefix=os.path.basename(args.output))
 
-        if len(data) == 0:
-            break
+    try:
+        for data in stream_iterator:
+            # We need to check if the player process still exists when
+            # using named pipes on Windows since the named pipe is not
+            # automatically closed by the player.
+            if is_win32 and is_fifo:
+                output.player.poll()
 
-        # We need to check if the player process still exists when
-        # using named pipes on Windows since the named pipe is not
-        # automatically closed by the player.
-        if is_win32 and is_fifo:
-            output.player.poll()
+                if output.player.returncode is not None:
+                    console.logger.info("Player closed")
+                    break
+
+            try:
+                output.write(data)
+            except IOError as err:
+                if is_player and err.errno in ACCEPTABLE_ERRNO:
+                    console.logger.info("Player closed")
+                elif is_http and err.errno in ACCEPTABLE_ERRNO:
+                    console.logger.info("HTTP connection closed")
+                else:
+                    console.logger.error("Error when writing to output: {0}", err)
 
-            if output.player.returncode is not None:
-                console.logger.info("Player closed")
                 break
-
-        try:
-            output.write(data)
-        except IOError as err:
-            if is_player and err.errno in ACCEPTABLE_ERRNO:
-                console.logger.info("Player closed")
-            elif is_http and err.errno in ACCEPTABLE_ERRNO:
-                console.logger.info("HTTP connection closed")
-            else:
-                console.logger.error("Error when writing to output: {0}",
-                                     err)
-
-            break
-
-        written += len(data)
-        prebuffer = None
-
-        if show_progress:
-            console.msg_inplace("Written {0} bytes", written)
-
-    if show_progress and written > 0:
-        console.msg_inplace_end()
+    except IOError as err:
+        console.logger.error("Error when reading from stream: {0}", err)
 
     stream.close()
     console.logger.info("Stream ended")
@@ -408,7 +400,7 @@ def fetch_streams_infinite(plugin, interval):
 def resolve_stream_name(streams, stream_name):
     """Returns the real stream name of a synonym."""
 
-    if stream_name in STREAM_SYNONYMS:
+    if stream_name in STREAM_SYNONYMS and stream_name in streams:
         for name, stream in streams.items():
             if stream is streams[stream_name] and name not in STREAM_SYNONYMS:
                 return name
@@ -673,6 +665,9 @@ def setup_options():
     if args.hls_segment_attempts:
         livestreamer.set_option("hls-segment-attempts", args.hls_segment_attempts)
 
+    if args.hls_segment_threads:
+        livestreamer.set_option("hls-segment-threads", args.hls_segment_threads)
+
     if args.hls_segment_timeout:
         livestreamer.set_option("hls-segment-timeout", args.hls_segment_timeout)
 
@@ -682,6 +677,18 @@ def setup_options():
     if args.hds_live_edge:
         livestreamer.set_option("hds-live-edge", args.hds_live_edge)
 
+    if args.hds_segment_attempts:
+        livestreamer.set_option("hds-segment-attempts", args.hds_segment_attempts)
+
+    if args.hds_segment_threads:
+        livestreamer.set_option("hds-segment-threads", args.hds_segment_threads)
+
+    if args.hds_segment_timeout:
+        livestreamer.set_option("hds-segment-timeout", args.hds_segment_timeout)
+
+    if args.hds_timeout:
+        livestreamer.set_option("hds-timeout", args.hds_timeout)
+
     if args.http_stream_timeout:
         livestreamer.set_option("http-stream-timeout", args.http_stream_timeout)
 
@@ -697,6 +704,18 @@ def setup_options():
     if args.rtmp_timeout:
         livestreamer.set_option("rtmp-timeout", args.rtmp_timeout)
 
+    if args.stream_segment_attempts:
+        livestreamer.set_option("stream-segment-attempts", args.stream_segment_attempts)
+
+    if args.stream_segment_threads:
+        livestreamer.set_option("stream-segment-threads", args.stream_segment_threads)
+
+    if args.stream_segment_timeout:
+        livestreamer.set_option("stream-segment-timeout", args.stream_segment_timeout)
+
+    if args.stream_timeout:
+        livestreamer.set_option("stream-timeout", args.stream_timeout)
+
     livestreamer.set_option("subprocess-errorlog", args.subprocess_errorlog)
 
     # Deprecated options
@@ -707,17 +726,9 @@ def setup_options():
 
 def setup_plugin_options():
     """Sets Livestreamer plugin options."""
-    if args.jtv_cookie:
-        livestreamer.set_plugin_option("justintv", "cookie",
-                                       args.jtv_cookie)
+    if args.twitch_cookie:
         livestreamer.set_plugin_option("twitch", "cookie",
-                                       args.jtv_cookie)
-
-    if args.jtv_password:
-        livestreamer.set_plugin_option("justintv", "password",
-                                       args.jtv_password)
-        livestreamer.set_plugin_option("twitch", "password",
-                                       args.jtv_password)
+                                       args.twitch_cookie)
 
     if args.twitch_oauth_token:
         livestreamer.set_plugin_option("twitch", "oauth_token",
@@ -756,6 +767,14 @@ def setup_plugin_options():
         console.logger.warning("The option --jtv/twitch-legacy-names is "
                                "deprecated and will be removed in the future.")
 
+    if args.jtv_cookie:
+        console.logger.warning("The option --jtv-cookie is deprecated and "
+                               "will be removed in the future.")
+
+    if args.jtv_password:
+        console.logger.warning("The option --jtv-password is deprecated "
+                               "and will be removed in the future.")
+
     if args.gomtv_username:
         console.logger.warning("The option --gomtv-username is deprecated "
                                "and will be removed in the future.")
@@ -817,10 +836,20 @@ def main():
     if args.plugins:
         print_plugins()
     elif args.url:
-        with ignored(KeyboardInterrupt):
+        try:
             setup_options()
             setup_plugin_options()
             handle_url()
+        except KeyboardInterrupt:
+            # Make sure current stream gets properly cleaned up
+            if stream_fd:
+                console.msg("Interrupted! Closing currently open stream...")
+                try:
+                    stream_fd.close()
+                except KeyboardInterrupt:
+                    sys.exit()
+            else:
+                console.msg("Interrupted! Exiting...")
     elif args.twitch_oauth_authenticate:
         authenticate_twitch_oauth()
     else:
